#include <AppKit/AppKit.h>
#include "WinDelegate.api.h"

static WinDelegate *sharedInstance;

@implementation WinDelegate

/*
 * The following a delegated notifications.
 * For each of them [notif object] provides the window.
 */
- (void) windowWillMove:(NSNotification *) notif
{
	NSWindow *window = [notif object];
	NSRect wFrame = [window frame];
	NSLog(@"winWillMove %@",
		NSStringFromPoint(wFrame.origin));
}

- (void) windowDidMove:(NSNotification *) notif
{
	NSWindow *window = [notif object];
	NSRect wFrame = [window frame];
	NSLog(@"winDidMove %@",
		NSStringFromPoint(wFrame.origin));
}

- (void) windowDidBecomeKey:(NSNotification *) notif
{
	NSLog(@"winDidBK");
}

- (void) windowDidResignKey:(NSNotification *) notif
{
	NSLog(@"winDidRK");
}

- (void) windowDidBecomeMain:(NSNotification *) notif
{
	NSLog(@"winDidBM");
}

- (void) windowDidResignMain:(NSNotification *) notif
{
	NSLog(@"winDidRM");
}

- (void) windowWillMiniaturize:(NSNotification *) notif
{
	NSLog(@"winWillMin");
}

- (void) windowDidMiniaturize:(NSNotification *) notif
{
	NSLog(@"winDidMin");
}

- (void) windowDidDeminiaturize:(NSNotification *) notif
{
	NSLog(@"winDidDemin");
}

- (void) windowWillClose:(NSNotification *) notif
{
	NSWindow *window = [notif object];
	NSLog(@"winWillClose %p", window);
	NSView *view = [window contentView];
	[window setContentView:nil];
	[view release];
}

- (void) windowDidChangeScreen:(NSNotification *) notif
{
	NSLog(@"winDidCS");
}

/* A portion of a non-retained window has been exposed. */
- (void) windowDidExpose:(NSNotification *) notif
{
	NSDictionary *dict = [notif userInfo];
	NSValue *value = [dict objectForKey: @"NSExposedRect"];
	NSRect rect = [value rectValue];
	NSLog(@"winDidEx %@", NSStringFromRect(rect));
}

- (void) windowDidResize:(NSNotification *) notif
{
	NSWindow *window = [notif object];
	NSRect wFrame = [window frame];
	NSView *content = [window contentView];
	NSRect cFrame = [content frame];
	NSLog(@"winDidResize c%@ w%@",
		NSStringFromSize(cFrame.size),
		NSStringFromSize(wFrame.size));
}

- (void) windowDidUpdate:(NSNotification *) notif
{
	NSLog(@"winDidUpdate");
}


/*
 * The following are direct delegations.
 */
- (BOOL) windowShouldClose:(id) sender
{
	NSLog(@"winShouldClose?");
	return YES;
}

- (NSSize) windowWillResize:(NSWindow *) sender toSize:(NSSize) frameSize
{
	NSLog(@"winWillResize %@", NSStringFromSize(frameSize));
	return frameSize;
}

#if 0 /* very noisy on osx */
- (id) windowWillReturnFieldEditor:(NSWindow *) sender toObject:(id) anObject
{
	NSLog(@"winWillReturnFieldEditor");
	return nil;
}
#endif

- (void) dealloc
{
	NSLog(@"winDelegateDealloc");
	[[NSNotificationCenter defaultCenter] removeObserver: self];
	[super dealloc];
}

+ (WinDelegate *) delegate
{
	if (!sharedInstance)
		sharedInstance = [[WinDelegate alloc] init];
	return sharedInstance;
}
@end
