#import <AppKit/AppKit.h>
#include "menu.h"

#define APP_NAME @"Zz"

#undef NSLocalizedString
#define NSLocalizedString(a,b) a

static void add_hide_item (NSMenu *menu, BOOL mac_style)
{
	NSMenuItem *item;
	NSString *name = @"";

	if (mac_style) {
		name = [NSString stringWithFormat:@" %@", APP_NAME];
	}
	item = [menu addItemWithTitle:
				[NSString stringWithFormat:@"%@%@",
					NSLocalizedString(@"Hide", nil), name]
			action:@selector(hide:)
			keyEquivalent:@"h"];
	[item setTarget:NSApp];
}

static void add_quit_item (NSMenu *menu, BOOL mac_style)
{
	NSMenuItem *item;

	NSString *name = @"";

	if (mac_style) {
		name = [NSString stringWithFormat:@" %@", APP_NAME];
	}
	item = [menu addItemWithTitle:
				[NSString stringWithFormat:@"%@%@",
					NSLocalizedString(@"Quit", nil), name]
			action:@selector(terminate:)
			keyEquivalent:@"q"];
	[item setTarget:NSApp];
}

static void populate_application_menu (NSMenu *menu, BOOL mac_style)
{
	NSMenuItem *item;
	NSString *name = @"";

	if (mac_style) {
		name = [NSString stringWithFormat:@" %@", APP_NAME];
	}


	/* About */
	item = [menu addItemWithTitle:
				[NSString stringWithFormat:@"%@%@",
					NSLocalizedString(@"About", nil), name]
			action:@selector(orderFrontStandardAboutPanel:)
			keyEquivalent:@""];
	[item setTarget:NSApp];
	if (mac_style)
		[menu addItem:[NSMenuItem separatorItem]];

	/* Preferences */
	item = [menu addItemWithTitle:NSLocalizedString(@"Preferences...", nil)
			action:NULL
			keyEquivalent:@","];

	if (mac_style) {
		/* Show/Hide */
		[menu addItem:[NSMenuItem separatorItem]];
		add_hide_item(menu, YES);
	}

	/* Show/Hide */
	item = [menu addItemWithTitle:NSLocalizedString(@"Hide Others", nil)
			action:@selector(hideOtherApplications:)
			keyEquivalent:@"h"];
	[item setKeyEquivalentModifierMask:NSCommandKeyMask|NSAlternateKeyMask];
	[item setTarget:NSApp];
	//
	item = [menu addItemWithTitle:NSLocalizedString(@"Show All", nil)
			action:@selector(unhideAllApplications:)
			keyEquivalent:@""];
	[item setTarget:NSApp];

	/* Quit */
	if (mac_style) {
		[menu addItem:[NSMenuItem separatorItem]];
		add_quit_item(menu, YES);
	}
}

static void add_application_menu (NSMenu *menu, BOOL mac_style)
{
	NSMenu *submenu;
	NSMenuItem *item;

	submenu = [NSMenu alloc];
	if (mac_style) {
		item = [menu addItemWithTitle:@"Apple"
				action:NULL
				keyEquivalent:@""];
		submenu = [submenu initWithTitle:@"Apple"];
		[NSApp performSelector:@selector(setAppleMenu:)
			withObject:submenu];
	} else {
		item = [menu addItemWithTitle:@"Info"
				action:NULL
				keyEquivalent:@""];
		submenu = [submenu initWithTitle:@"Info"];
	}
	populate_application_menu(submenu, mac_style);
	[menu setSubmenu:submenu forItem:item];
}

static void add_front_item (NSMenu *menu, BOOL mac_style)
{
	NSMenuItem *item;

	NSString *name;

	if (mac_style)
		name = NSLocalizedString(@"Bring All to Front", nil);
	else
		name = NSLocalizedString(@"Arrange in Front", nil);
	item = [menu addItemWithTitle:name
			action:@selector(arrangeInFront:)
			keyEquivalent:@""];
}

static void populate_windows_menu (NSMenu *menu, BOOL mac_style)
{
	NSMenuItem *item;

	if (!mac_style)
		add_front_item(menu, NO);
	item = [menu addItemWithTitle:NSLocalizedString(@"Minimize", nil)
			action:@selector(performMiniaturize:)
			keyEquivalent:@"m"];
	item = [menu addItemWithTitle:NSLocalizedString(@"Zoom", nil)
			action:@selector(performZoom:)
			keyEquivalent:@""];
	if (mac_style)
		[menu addItem:[NSMenuItem separatorItem]];

	item = [menu addItemWithTitle:NSLocalizedString(@"Previous Window", nil)
			action:NULL
			keyEquivalent:@"`"];
	item = [menu addItemWithTitle:NSLocalizedString(@"Next Window", nil)
			action:NULL
			keyEquivalent:@"~"];

	if (mac_style) {
		[menu addItem:[NSMenuItem separatorItem]];
		add_front_item(menu, YES);
	}
}

static void add_windows_menu (NSMenu *menu, BOOL mac_style)
{
	NSMenu *submenu;
	NSMenuItem *item;

	NSString *name;

	name = [NSString stringWithFormat:@"Window%s",
				mac_style ? "" : "s"];
	item = [menu addItemWithTitle:name
			action:NULL
			keyEquivalent:@""];
	submenu = [[NSMenu alloc] initWithTitle:NSLocalizedString(name, nil)];
	populate_windows_menu(submenu, mac_style);
	[menu setSubmenu:submenu forItem:item];
	[NSApp setWindowsMenu:submenu];

}

static void main_menu (BOOL mac_style)
{
	NSMenu *menu;

	menu = [NSMenu alloc];

	if (mac_style)
		menu = [menu initWithTitle:@"MainMenu"];
	else
		menu = [menu initWithTitle:APP_NAME];

	add_application_menu(menu, mac_style);

	add_windows_menu(menu, mac_style);

	if (!mac_style) {
		add_hide_item(menu, NO);
		add_quit_item(menu, NO);
	}

	[NSApp setMainMenu: menu];
}

void create_menus (void)
{
#if defined(__APPLE__)
	main_menu(YES);
#else
	main_menu(NO);
#endif
}
